@@ -9,8 +9,8 @@ ConfigurationType = Dict[str, Union[List, Dict[str, Any]]]
 GetItemType = Union[str, FactoryType]
 
 
-def configure(config: Union[str, ConfigurationType]):
-    Registry.default_registry().configure(config)
+def configure(config: Union[str, ConfigurationType], file_type: Optional[str] = None):
+    Registry.default_registry().configure(config, file_type)
 
 
 def get(item: GetItemType, *args: Any, **kwargs: Any) -> Any:
